package com.yixin.ms.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.excel.util.CollectionUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.beust.jcommander.internal.Lists;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.boot.restful.RestResponse;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.DESUtil;
import com.yixin.ms.boot.uitls.OkHttpUtils;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.boot.uitls.SMSOrIdCardUtils;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.HealthCodeLevelEnum;
import com.yixin.ms.model.enums.HealthVaccineNumEnum;
import com.yixin.ms.model.enums.IDTypeEnum;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.PlaceHealthCodeServer;
import com.yixin.ms.server.dto.DeviceScanCodeDTO;
import com.yixin.ms.server.dto.DeviceScanIdCardDTO;
import com.yixin.ms.server.dto.PlaceRecordDTO;
import com.yixin.ms.server.vo.DeviceScanIdCardVO;
import com.yixin.ms.server.vo.HealthResultVO;
import com.yixin.ms.server.vo.PlaceInfoVO;
import com.yixin.ms.service.HealthCodeService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executor;


@Slf4j
@Service
public class HealthCodeServiceImpl implements HealthCodeService {
    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private PlaceHealthCodeServer placeHealthCodeServer;
    @Autowired
    private Executor executor;

    @Override
    public HealtCodeUserVO getShortTermCertificate(ShortTermDTO shortTermDTO) {
        try {
            HealtCodeUserVO shortTermUser = getShortTermUser(shortTermDTO.getIdNumber(), shortTermDTO.getName());
            if (shortTermUser != null && !CollectionUtils.isEmpty(shortTermUser.getData())) {
                return shortTermUser;
            }
            return getSjJkm(shortTermDTO.getIdNumber(), shortTermDTO.getName());
        } catch (Exception e) {
            e.printStackTrace();
            log.info("异常 :" + e.getMessage());
            try {
                return getSjJkm(shortTermDTO.getIdNumber(), shortTermDTO.getName());
            } catch (Exception ex) {
                ex.printStackTrace();
                throw new ServiceException("未获取到健康码信息，请联系工作人员");
            }
        }

    }

    @Override
    public String addToken(String token) {
        String s = "hh_token_" + UUID.randomUUID();
        boolean hanghui = redisUtil.set(s, s);
        if (hanghui) {
            return s;
        } else {
            throw new ServiceException("体检失败");
        }

    }

    @Override
    public Boolean deleteToken(String token) {
        return redisUtil.delete(token);
    }

    @Override
    public RestResult getShortTerm(ShortTermDTO shortTermDTO) {
        if (StrUtil.isEmpty(shortTermDTO.getHealthCode())) {
            if (StringUtils.isBlank(shortTermDTO.getIdNumber()) && StringUtils.isBlank(shortTermDTO.getName())) {
                throw new ServiceException(701, "身份证和姓名为空哦");
            }
        }

        if(StrUtil.isNotEmpty(shortTermDTO.getHealthCode()) && (HttpUtil.isHttp(shortTermDTO.getHealthCode()) || HttpUtil.isHttps(shortTermDTO.getHealthCode()))){
            log.info("请求健康码参数信息： {}", shortTermDTO.getHealthCode());
            //扫码推送参数为链接，提取杭州健康码barCode参数信息
            String barCode = HttpUtil.decodeParamMap(shortTermDTO.getHealthCode().toLowerCase(), Charset.defaultCharset()).get("statecouncilbarcode");
            if(StrUtil.isEmpty(barCode)){
                throw new ServiceException("请打开支付宝选择杭州健康码重试");
            }else {
                shortTermDTO.setHealthCode(barCode);
            }
        }

        if (StringUtils.isBlank(shortTermDTO.getToken())) {
            throw new ServiceException(700, "token为空哦");
        }
        if (StringUtils.isNotBlank(shortTermDTO.getIdNumber())) {
            if (!SMSOrIdCardUtils.isIdCard(shortTermDTO.getIdNumber())) {
                throw new ServiceException(704, "身份证格式有误");
            }
        }

        HealtCodeUserVO userVO = null;
        if (StringUtils.isNotEmpty(shortTermDTO.getSn())) {
            RestResult<PlaceInfoVO> placeInfoResult = placeHealthCodeServer.queryPlaceInfoBySn(shortTermDTO.getSn());
            if (placeInfoResult != null && placeInfoResult.getData() != null && StrUtil.isEmpty(placeInfoResult.getData().getAddressCode()) && RestCode.SUCCESS.getCode() == placeInfoResult.getCode() && StrUtil.isNotEmpty(shortTermDTO.getHealthCode())) {
                throw new ServiceException("设备扫码未绑定场所码，请联系管理员");
            }

            if (placeInfoResult != null && placeInfoResult.getData() != null && StrUtil.isNotEmpty(placeInfoResult.getData().getAddressCode()) && RestCode.SUCCESS.getCode() == placeInfoResult.getCode()) {
                PlaceInfoVO placeInfoVO = placeInfoResult.getData();
                HealthResultVO<DeviceScanIdCardVO> scanIdCardResult = null;
                if (StrUtil.isNotEmpty(shortTermDTO.getHealthCode())) {
                    //健康码查询场所码信息
                    scanIdCardResult = placeHealthCodeServer.deviceScanCode(new DeviceScanCodeDTO().setDeviceId(shortTermDTO.getSn()).setHealthCodeQrCode(shortTermDTO.getHealthCode()).setUrlParams(shortTermDTO.getHealthCode()));
                    //长时间不调用会提示设备未打卡错误，重新发起调用
                    if (scanIdCardResult == null || !scanIdCardResult.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
                        if (scanIdCardResult != null && scanIdCardResult.getCode().equals(Integer.valueOf(RestCode.INVALID_TOKEN.getCode()))) {
                            //清理token缓存
                            placeHealthCodeServer.getToken(true);
                        }

                        try {
                            Thread.sleep(500);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                        scanIdCardResult = placeHealthCodeServer.deviceScanCode(new DeviceScanCodeDTO().setDeviceId(shortTermDTO.getSn()).setHealthCodeQrCode(shortTermDTO.getHealthCode()).setUrlParams(shortTermDTO.getHealthCode()));
                    }
                } else {
                    //扫人员证件查询场所码信息
                    scanIdCardResult = placeHealthCodeServer.deviceScanIdCard(new DeviceScanIdCardDTO().setDeviceId(shortTermDTO.getSn()).setIdNumber(shortTermDTO.getIdNumber()).setIdType(IDTypeEnum.ID_CARD.getCode()));
                    //长时间不调用会提示设备未打卡错误，重新发起调用
                    if (scanIdCardResult == null || !scanIdCardResult.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
                        if (scanIdCardResult != null && scanIdCardResult.getCode().equals(Integer.valueOf(RestCode.INVALID_TOKEN.getCode()))) {
                            //清理token缓存
                            placeHealthCodeServer.getToken(true);
                        }

                        try {
                            Thread.sleep(500);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                        scanIdCardResult = placeHealthCodeServer.deviceScanIdCard(new DeviceScanIdCardDTO().setDeviceId(shortTermDTO.getSn()).setIdNumber(shortTermDTO.getIdNumber()).setIdType(IDTypeEnum.ID_CARD.getCode()));
                    }
                }

                if (scanIdCardResult == null || !scanIdCardResult.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
                    //场所码查询若不成功走之前的
                    userVO = this.getShortTermCertificate(shortTermDTO);
                } else {
                    DeviceScanIdCardVO scanIdCardVO = scanIdCardResult.getData();
                    userVO = new HealtCodeUserVO();
                    //疫苗接种数量
                    userVO.setVaccineNum(scanIdCardVO.getVaccinated());
                    userVO.setNuclein(scanIdCardVO.getReportTime() == null ? Lists.newArrayList() : Lists.newArrayList(new HealtCodeNucleinVO().setChecktime(DateUtil.parseDateTime(scanIdCardVO.getReportTime())).setResult(scanIdCardVO.getCheckResult())));
                    userVO.setData(Lists.newArrayList(new HealtCodeTypeVO().setMzt(HealthCodeLevelEnum.codeOf(scanIdCardVO.getLevel()).getDesc())));
                    //行程信息
                    userVO.setTravelValidation(scanIdCardVO.getTravelValidation());
                    userVO.setUsername(StrUtil.isEmpty(shortTermDTO.getName()) ? scanIdCardVO.getUsername() : shortTermDTO.getName());
                    PlaceRecordDTO placeRecordDTO = new PlaceRecordDTO().setName(StrUtil.isEmpty(shortTermDTO.getName()) ? scanIdCardVO.getUsername() : shortTermDTO.getName()).setIdNumber(shortTermDTO.getIdNumber()).setLevel(HealthCodeLevelEnum.codeOf(scanIdCardVO.getLevel()).getDesc()).setCheckTime(scanIdCardVO.getCheckTime() == null ? null : DateUtil.parseDateTime(scanIdCardVO.getCheckTime())).setReportTime(scanIdCardVO.getReportTime() == null ? null : DateUtil.parseDateTime(scanIdCardVO.getReportTime())).setCheckResult(scanIdCardVO.getCheckResult()).setTravelValidation(scanIdCardVO.getTravelValidation()).setVaccinated(scanIdCardVO.getVaccinated()).setAddressCode(placeInfoVO.getAddressCode()).setUnitName(placeInfoVO.getUnitName()).setUnitAddress(placeInfoVO.getUnitAddress()).setManagerHealthCode(placeInfoVO.getManagerHealthCode()).setHealthCode(shortTermDTO.getHealthCode()).setAdminId(placeInfoVO.getAdminId()).setCreateTime(DateUtil.date());
                    executor.execute(() -> {
                        //保存场所记录
                        placeHealthCodeServer.savePlaceRecord(placeRecordDTO);
                    });
                }
            } else {
                //之前的健康码查询
                userVO = this.getShortTermCertificate(shortTermDTO);
            }
        } else {
            //设备sn为空默认走之前的健康码查询
            userVO = this.getShortTermCertificate(shortTermDTO);
        }

        if (userVO != null) {
            if (userVO.getData() == null || userVO.getData().isEmpty()) {
                throw new ServiceException("未查询到健康吗");
            }

            //姓名
            userVO.setUsername(StrUtil.isEmpty(userVO.getUsername()) ? shortTermDTO.getName() : userVO.getUsername());
            //身份证号
            userVO.setIdNumber(StrUtil.isEmpty(userVO.getIdNumber()) ? shortTermDTO.getIdNumber() : userVO.getIdNumber());

            /**
             * 0 未接种 对应0针
             *
             * 1-接种完成 对应2针
             *
             * 2-未完全接种 对应1针
             *
             * 3- 完成加强针 对应3针
             */
            if(HealthVaccineNumEnum.VACCINATION_COMPLETE.getStatus().equals(userVO.getVaccineNum())){
                userVO.setVaccineNum(HealthVaccineNumEnum.VACCINATION_COMPLETE.getNum());
            }else if(HealthVaccineNumEnum.INCOMPLETE_INOCULATION.getStatus().equals(userVO.getVaccineNum())){
                userVO.setVaccineNum(HealthVaccineNumEnum.INCOMPLETE_INOCULATION.getNum());
            }else{
                userVO.setVaccineNum(userVO.getVaccineNum());
            }

            return RestResponse.ok(userVO);
        }

        throw new ServiceException(703, "未查询到此数据");
    }

    public HealtCodeUserVO getShortTermUser(String sfzh, String name) throws Exception {
        log.info("江干分局健康码传参，身份证{} 姓名 {}", sfzh, name);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("sfzh", sfzh);
        jsonObject.put("xm", name);
        String sync = OkHttpUtils.builder().url("http://115.236.38.205:8903/id/hvn")
                .addHeader("Content-Type", "application/json; charset=utf-8")
                .localPost(jsonObject.toJSONString())
                .sync();
        if (sync.contains("请求失败：Connect timed out")) {
            throw new ServiceException("江干分局请重试");
        }
        log.info("江干分局健康码返回结果，{}", sync);
        HealtCodeUserVO healtCodeUserVO = new HealtCodeUserVO();
        if (StringUtils.isNotBlank(sync)) {
            JSONObject json = JSON.parseObject(sync);
            if (StringUtils.equals("00", json.getString("error_code"))) {
                healtCodeUserVO.setData(JSON.parseArray(json.getJSONArray("data").toJSONString(), HealtCodeTypeVO.class));
                healtCodeUserVO.setNuclein(JSON.parseArray(json.getJSONArray("nuclein").toJSONString(), HealtCodeNucleinVO.class));
                healtCodeUserVO.setVaccineNum(json.getString("vaccine_num"));
            } else {
                throw new ServiceException("江干分局异常:" + jsonObject.getString("msg"));
            }
        }
        return healtCodeUserVO;
    }


    public HealtCodeUserVO getSjJkm(String sfzh, String name) throws Exception {
        log.info("市局健康码传参，身份证{} 姓名 {}", sfzh, name);
        String token = "";
        token = (String) redisUtil.get("jkm-token");
        if (StringUtils.isBlank(token)) {
            token = getToken();
            if (StringUtils.isBlank(token)) {
                throw new ServiceException("市局健康码请重试");
            }
        }
        JSONObject data = new JSONObject();
        JSONObject body = new JSONObject();
        body.put("idCard", sfzh);
        String encryptSfzh = DESUtil.encrypt(body.toJSONString(), "7760ce65a8544f779f36607c201e8a0c");
        data.put("body", encryptSfzh);
        data.put("token", token);
        String sync = OkHttpUtils.builder().url("http://policecard.tenzont.com/auth/api/healthCode")
                .addHeader("Content-Type", "application/json; charset=utf-8")
                .localPost(data.toJSONString())
                .sync();
        if (sync.contains("请求失败：Connect timed out")) {
            throw new ServiceException("市局健康码请重试");
        }
        log.info("市局健康码返回结果，{}", sync);
        JSONObject jkm = JSONObject.parseObject(sync);
        String rspCode = jkm.getJSONObject("header").getString("rspCode");
        HealtCodeUserVO healtCodeUserVO = new HealtCodeUserVO();
        if (StringUtils.equals("00", rspCode)) {
            String mzt = jkm.getJSONObject("body").getString("mzt");
            if (StringUtils.isNotBlank(mzt)) {
                List<HealtCodeTypeVO> list = new ArrayList<>();
                HealtCodeTypeVO healtCodeTypeVO = new HealtCodeTypeVO();
                healtCodeTypeVO.setMzt(mzt);
                list.add(healtCodeTypeVO);
                healtCodeUserVO.setData(list);
                healtCodeUserVO.setNuclein(new ArrayList<>());
                healtCodeUserVO.setVaccineNum("");
                return healtCodeUserVO;
            } else {
                throw new ServiceException("市局健康码请重试");
            }
        } else if (StringUtils.equals("01", rspCode)) {
            getToken();
            HealtCodeUserVO sjJkm = getSjJkm(sfzh, name);
            return sjJkm;
        } else {
            throw new ServiceException("市局健康码请重试");
        }
    }


    public String getToken() throws Exception {
        String sync = "";
        JSONObject jsonObject = new JSONObject();
        JSONObject body = new JSONObject();
        body.put("appid", "jzz3c086d786420a958a143e6154a351");
        body.put("secret", "ecbe0e220ec74d05b2b22268a737d2f1");
        jsonObject.put("body", body);
        try {
            sync = OkHttpUtils
                    .builder()
                    .url("http://policecard.tenzont.com/auth/api/token")
                    .localPost(jsonObject.toJSONString())
                    .sync();
        } catch (Exception e) {
            log.info("获取健康码的token接口异常 ，{}", e.getMessage());
            return "";
        }
        if (StringUtils.isBlank(sync)) {
            return "";
        }
        JSONObject fhz = JSONObject.parseObject(sync);
        String rspCode = fhz.getJSONObject("header").getString("rspCode");
        if (StringUtils.equals("00", rspCode)) {
            String token = fhz.getJSONObject("body").getString("token");
            if (StringUtils.isNotBlank(token)) {
                redisUtil.set("jkm-token", token, 7200);
                return token;
            }
        }
        return "";
    }

//    public static void main(String[] args) throws Exception {
//        System.out.println(getSjJkm("412728199706030513", ""));
//    }
}

